///////////////////////////////////////////////////////////////////////////////
// cyfoncon.c  Bitmap to fixed-width font converter
// by Athlor   Based on Ernest Pazera's FontMake code
///////////////////////////////////////////////////////////////////////////////
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <commctrl.h>
#include <tchar.h>

static LRESULT CALLBACK MainDlgProc(HWND,UINT,WPARAM,LPARAM);
int SaveFont(int);

static HANDLE ghInstance;

OPENFILENAME OFN;
char Bname[32]="";
char Fname[32]="";
char szFileName[MAX_PATH]="";

///////////////////////////////////////////////////////////////////////////////
int PASCAL WinMain(HINSTANCE hInst,HINSTANCE hPInst,LPSTR CL,int CS)
{
  WNDCLASSEX wcx;
  ghInstance = hInst;
  InitCommonControls();
  wcx.cbSize=sizeof(wcx);
  if(!GetClassInfoEx(NULL,MAKEINTRESOURCE(32770),&wcx)) return 0;
  wcx.hInstance=hInst;
  wcx.hIcon=LoadIcon(hInst,MAKEINTRESOURCE(8001));
  wcx.lpszClassName= _T("CyFontWiClass");
  if(!RegisterClassEx(&wcx)) return 0;
  return DialogBox(hInst,MAKEINTRESOURCE(1001),NULL,(DLGPROC)MainDlgProc);
}

///////////////////////////////////////////////////////////////////////////////
static LRESULT CALLBACK MainDlgProc(HWND hDlg,UINT uMsg,WPARAM wP,LPARAM lP)
{
  switch (uMsg)
  {
    case WM_COMMAND:
      switch (GET_WM_COMMAND_ID(wP, lP))
      {
        case 4007: // Browse Button
        {
          ZeroMemory(&OFN, sizeof(OFN));
          OFN.lStructSize = sizeof(OPENFILENAME);
          OFN.hwndOwner = hDlg;
          OFN.lpstrFilter = "Bitmaps (*.bmp)\0*.bmp\0All Files (*.*)\0*.*\0";
          OFN.lpstrFile = szFileName; // Long filename
          OFN.lpstrFileTitle = Bname; // Short filename
          OFN.nMaxFileTitle = 32;     // Short filename's max length
          OFN.nMaxFile = MAX_PATH;
          OFN.Flags = OFN_EXPLORER|OFN_FILEMUSTEXIST|OFN_HIDEREADONLY;
          OFN.lpstrDefExt = "bmp";
          if (GetOpenFileName(&OFN))
          {
            SetDlgItemText(hDlg,4001,Bname);
            strcpy(Fname,Bname);                 // Copy bmp name to font name
            memcpy(strrchr(Fname,'.'),".fnt",4); // Change type on font name
            SetDlgItemText(hDlg,4002,Fname);
          }
        }
          return TRUE;
        case 4003: // Convert to 96 Button
          SaveFont(96);
          return TRUE;
        case 4004: // Convert to 224 Button
          SaveFont(224);
          return TRUE;
        case IDOK:
          EndDialog(hDlg, TRUE);
          return TRUE;
      }
      break;
    case WM_CLOSE:
      EndDialog(hDlg, 0);
      return TRUE;
  }
  return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
int SaveFont(int len)
{
 int H=6;
  if(len==224) H=14;
  HDC hdc=CreateCompatibleDC(NULL);
  HBITMAP hbmNew=(HBITMAP)LoadImage(NULL,Bname,IMAGE_BITMAP,0,0,LR_LOADFROMFILE);
  HBITMAP hbmOld=(HBITMAP)SelectObject(hdc,hbmNew);
  BITMAP bmp;

  GetObject(hbmNew,sizeof(BITMAP),&bmp);

  int CharW=bmp.bmWidth/16;
  int CharH=bmp.bmHeight/H;
  int CellW=((CharW+7)/8)*8;

  HDC hdcChar=CreateCompatibleDC(NULL);
  HBITMAP hbmChar=CreateCompatibleBitmap(hdc,CellW,CharH);
  HBITMAP hbmCharOld=(HBITMAP)SelectObject(hdcChar,hbmChar);

  unsigned char* buffer;
  int filesize=((CellW/8)*CharH+4)*len+4; // (# of chars = 96 or 224)

  buffer=GlobalAlloc(GPTR,filesize+1); // buffer=new unsigned char[filesize];
  int fileindex=0;

  buffer[fileindex++]=0x01; // Font Header
  buffer[fileindex++]=0x60;
  buffer[fileindex++]=CharW;
  buffer[fileindex++]=CharH;

  unsigned char pel;
  int cellx,celly,pelx,pely,bytex;

  for(celly=0;celly<H;celly++)
  {
    for(cellx=0;cellx<16;cellx++)
    {
      for(pely=0;pely<CharH;pely++) // Clear out cell
      {
        for(pelx=0;pelx<CellW;pelx++)
        {
          SetPixelV(hdcChar,pelx,pely,RGB(255,255,255));
        }
      } // Copy cell
      BitBlt(hdcChar,0,0,CharW,CharH,hdc,cellx*CharW,celly*CharH,SRCCOPY);
      buffer[fileindex++]=0x00;
      buffer[fileindex++]=0x00;
      buffer[fileindex++]=CharW;
      buffer[fileindex++]=CharH;
      for(pely=0;pely<CharH;pely++)
      {
        for(bytex=0;bytex<(CellW/8);bytex++)
        {
          pel=0;
          for(pelx=0;pelx<8;pelx++)
          {
            if(GetPixel(hdcChar,bytex*8+pelx,pely)==RGB(0,0,0))
              pel|=(1<<(7-pelx));
          }
          buffer[fileindex++]=pel;
        }
      }
    }
  }

  HANDLE hfile=CreateFile(Fname,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
  DWORD numwritten;

  WriteFile(hfile,buffer,filesize,&numwritten,NULL);
  CloseHandle(hfile);
  GlobalFree(buffer); // delete [] buffer;
  SelectObject(hdcChar,hbmCharOld);
  DeleteObject(hbmChar);
  DeleteDC(hdcChar);
  SelectObject(hdc,hbmOld);
  DeleteObject(hbmNew);
  DeleteDC(hdc);
  char m[64];
  wsprintf(m,"%d char font in %d by %d format created",len,CharW,CharH);
  MessageBox(NULL,m,"CyFonCon",MB_OK);
 return 1;
}

